@@ -1,5 +1,3 @@
-package org.debian.maven.plugin;
-
 /*
  * Copyright 2009 Torsten Werner, Ludovic Claude.
  *
@@ -16,12 +14,15 @@ package org.debian.maven.plugin;
  * limitations under the License.
  */
 
+package org.debian.maven.plugin;
+
 import java.io.File;
 import java.io.FileReader;
 import java.io.IOException;
 import java.util.ArrayList;
 import java.util.List;
 import java.util.Properties;
+import java.util.regex.Pattern;
 
 import org.apache.maven.plugin.AbstractMojo;
 import org.apache.maven.plugin.MojoExecutionException;
@@ -36,6 +37,12 @@ import org.debian.maven.repo.POMCleaner;
  */
 public class SysInstallMojo extends AbstractMojo {
 
+    /** Regex for detecting that package is a libXXX-java package */
+    private static final Pattern JAVA_LIB_REGEX = Pattern.compile("lib.*-java");
+
+    /** Regex for detecting that package is a maven plugin package */
+    private static final Pattern PLUGIN_REGEX = Pattern.compile("lib.*-maven-plugin-java|libmaven-.*-plugin-java");
+
     // ----------------------------------------------------------------------
     // Mojo parameters
     // ----------------------------------------------------------------------
@@ -145,6 +152,18 @@ public class SysInstallMojo extends AbstractMojo {
      */
     private String mavenRules;
 
+    /**
+     * @parameter expression="${maven.ignoreRules}" default-value="maven.ignoreRules"
+     * @required
+     */
+    private String mavenIgnoreRules;
+
+    /**
+     * @parameter expression="${maven.publishedRules}" default-value="maven.publishedRules"
+     * @required
+     */
+    private String mavenPublishedRules;
+
     /**
      * root directory of the Maven repository
      *
@@ -177,6 +196,7 @@ public class SysInstallMojo extends AbstractMojo {
 
     private String classifier;
 
+
     // ----------------------------------------------------------------------
     // Public methods
     // ----------------------------------------------------------------------
@@ -184,8 +204,7 @@ public class SysInstallMojo extends AbstractMojo {
     public void execute() throws MojoExecutionException {
         try {
             runMojo();
-        }
-        catch (IOException e) {
+        } catch (IOException e) {
             getLog().error("execution failed", e);
             throw new MojoExecutionException("IOException catched");
         }
@@ -299,6 +318,22 @@ public class SysInstallMojo extends AbstractMojo {
         this.mavenRules = mavenRules;
     }
 
+    public String getMavenIgnoreRules() {
+        return mavenIgnoreRules;
+    }
+
+    public void setMavenIgnoreRules(String mavenIgnoreRules) {
+        this.mavenIgnoreRules = mavenIgnoreRules;
+    }
+
+    public String getMavenPublishedRules() {
+        return mavenPublishedRules;
+    }
+
+    public void setMavenPublishedRules(String mavenPublishedRules) {
+        this.mavenPublishedRules = mavenPublishedRules;
+    }
+
     public File getRepoDir() {
         return repoDir;
     }
@@ -535,8 +570,7 @@ public class SysInstallMojo extends AbstractMojo {
      * command for creating the relative symlink
      */
     private String[] linkCommand(String source, String dest) {
-        String[] command = {"ln", "-s", source, dest};
-        return command;
+        return new String[]{"ln", "-s", source, dest};
     }
 
     private void mkdir(String path) throws IOException {
@@ -658,19 +692,37 @@ public class SysInstallMojo extends AbstractMojo {
                 classifier = pomOption.getClassifier();
             }
 
-            installToUsj = pomOption.isJavaLib();
+            // If package is a java libary (lib.*-java)and is not a maven plugin
+            // (lib.*-plugin-java) potentially install to /usr/share/java
+            boolean packageIsJavaLib =
+                    JAVA_LIB_REGEX.matcher(destPackage).matches() &&
+                   !PLUGIN_REGEX.matcher(destPackage).matches();
+
+            // Its also possible to configure USJ install using
+            // DEB_MAVEN_INSTALL_TO_USJ which we should honour
+            // over auto-detection if its set to 'false'.
+            // This is stored in 'installToUsj' on class instance
+            // creation
+            installToUsj = pomOption.isJavaLib() ||
+                           (packageIsJavaLib && installToUsj);
         }
 
-        List params = new ArrayList();
+        List<String> params = new ArrayList<String>();
         params.add("--keep-pom-version");
 
         params.add("--package=" + destPackage);
         String mavenRulesPath = new File(debianDir, mavenRules).getAbsolutePath();
         params.add("--rules=" + mavenRulesPath);
+        String mavenIgnoreRulesPath = new File(debianDir, mavenIgnoreRules).getAbsolutePath();
+        params.add("--ignore-rules=" + mavenIgnoreRulesPath);
+        String mavenPublishedRulesPath = new File(debianDir, mavenPublishedRules).getAbsolutePath();
+        params.add("--published-rules=" + mavenPublishedRulesPath);
 
         System.out.println("Cleaning pom file: " + pomFile + " with options:");
         System.out.println("\t--keep-pom-version --package=" + destPackage);
         System.out.println("\t--rules=" + mavenRulesPath);
+        System.out.println("\t--ignore-rules=" + mavenIgnoreRulesPath);
+        System.out.println("\t--published-rules=" + mavenPublishedRulesPath);
 
         // add optional --no-parent option
         if (pomOption != null && pomOption.isNoParent()) {
@@ -688,7 +740,7 @@ public class SysInstallMojo extends AbstractMojo {
         params.add(cleanedPomSrcPath());
         params.add(cleanedPomPropertiesSrcPath());
 
-        POMCleaner.main((String[]) params.toArray(new String[params.size()]));
+        POMCleaner.main(params.toArray(new String[params.size()]));
 
         Properties pomProperties = new Properties();
         try {
@@ -707,7 +759,7 @@ public class SysInstallMojo extends AbstractMojo {
             params.add(debianPomSrcPath());
             params.add(debianPomPropertiesSrcPath());
 
-            POMCleaner.main((String[]) params.toArray(new String[params.size()]));
+            POMCleaner.main(params.toArray(new String[params.size()]));
         }
     }
 
