@@ -17,17 +17,22 @@
 
 package org.debian.maven.packager;
 
+import java.io.BufferedReader;
 import java.io.File;
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.InputStreamReader;
-import java.io.LineNumberReader;
 import java.util.*;
 import java.util.logging.Level;
 import java.util.logging.Logger;
+import java.util.regex.Matcher;
+import java.util.regex.Pattern;
 
 import javax.xml.stream.XMLStreamException;
 
+import org.debian.maven.packager.interaction.ChoiceQuestion;
+import org.debian.maven.packager.interaction.SimpleQuestion;
+import org.debian.maven.packager.interaction.YesNoQuestion;
 import org.debian.maven.packager.util.*;
 import org.debian.maven.repo.Dependency;
 import org.debian.maven.repo.DependencyNotFoundException;
@@ -56,7 +61,6 @@ import static org.debian.maven.repo.DependencyRuleSetFiles.RulesType.*;
 public class DependenciesSolver {
 
     private static final Logger log = Logger.getLogger(DependenciesSolver.class.getName());
-    private final UserInteraction userInteraction = new UserInteraction();
     private final IgnoreDependencyQuestions ignoreDependencyQuestion;
 
     private File baseDir;
@@ -83,12 +87,16 @@ public class DependenciesSolver {
     private boolean filterModules = false;
     boolean verbose = false;
     private Map<String, POMInfo> pomInfoCache = new HashMap<String, POMInfo>();
-    // Keep the original POMs for reference
+
+    /** The original POMs for reference */
     private Map<String, POMInfo> originalPomInfoCache = new HashMap<String, POMInfo>();
-    // Keep the previous selected rule for a given version
+
+    /** Map of the previously selected rule for a given version */
     private Map<String, Rule> versionToRules = new HashMap<String, Rule>();
-    // Keep the list of packages and dependencies
+
+    /** List of packages and dependencies */
     private Map<DebianDependency, Dependency> versionedPackagesAndDependencies = new HashMap<DebianDependency, Dependency>();
+
     private List<Rule> defaultRules = new ArrayList<Rule>();
     private PackageScanner scanner;
 
@@ -96,27 +104,25 @@ public class DependenciesSolver {
         this.outputDirectory = outputDirectory;
         this.scanner = scanner;
         this.interactive = interactive;
-        this.ignoreDependencyQuestion = new IgnoreDependencyQuestions(userInteraction, interactive);
+        this.ignoreDependencyQuestion = new IgnoreDependencyQuestions(interactive);
         pomTransformer.setVerbose(true);
         pomTransformer.setFixVersions(false);
         pomTransformer.setRulesFiles(initDependencyRuleSetFiles(outputDirectory, verbose));
 
-        Rule toDebianRule = new Rule("s/.*/debian/", "Change the version to the symbolic 'debian' version");
-        Rule keepVersionRule = new Rule("*", "Keep the version");
-        Rule customRule = new Rule("CUSTOM", "Custom rule");
-        defaultRules.add(toDebianRule);
-        defaultRules.add(keepVersionRule);
-        defaultRules.add(customRule);
+        defaultRules.add(new Rule("s/.*/debian/", "Change the version to the symbolic 'debian' version"));
+        defaultRules.add(new Rule("*", "Keep the version"));
+        defaultRules.add(new Rule("CUSTOM", "Custom rule"));
     }
 
     public static DependencyRuleSetFiles initDependencyRuleSetFiles(File outputDirectory, boolean verbose) {
         DependencyRuleSetFiles depFiles = new DependencyRuleSetFiles();
-        for(RulesType type : RulesType.values()) {
-            if(type.filename == null) continue;
-            File rulesFile = new File(outputDirectory, type.filename);
-            String description = readResource(type.descriptionResource);
-            DependencyRuleSet ruleSet = DependencyRuleSet.readRules(rulesFile, description, verbose, false);
-            depFiles.get(type).addAll(ruleSet);
+        for (RulesType type : RulesType.values()) {
+            if (type.filename != null) {
+                File rulesFile = new File(outputDirectory, type.filename);
+                String description = readResource(type.descriptionResource);
+                DependencyRuleSet ruleSet = DependencyRuleSet.readRules(rulesFile, description, verbose, false);
+                depFiles.get(type).addAll(ruleSet);
+            }
         }
 
         return depFiles;
@@ -127,7 +133,7 @@ public class DependenciesSolver {
         StringBuilder sb = new StringBuilder();
         try {
             InputStream is = DependenciesSolver.class.getResourceAsStream("/" + resource);
-            LineNumberReader r = new LineNumberReader(new InputStreamReader(is));
+            BufferedReader r = new BufferedReader(new InputStreamReader(is));
             String line;
             while ((line = r.readLine()) != null) {
                 sb.append(line);
@@ -314,7 +320,7 @@ public class DependenciesSolver {
             getRepository().registerPom(projectPom, pom);
             // Also register automatically the test jar which may accompany the current jar and be
             // used in another module of the same project
-            if ("jar".equals(pom.getThisPom().getType())) {
+            if (pom.getThisPom().isJar()) {
                 POMInfo testPom = (POMInfo) pom.clone();
                 testPom.getThisPom().setType("test-jar");
                 getRepository().registerPom(projectPom, testPom);
@@ -323,8 +329,8 @@ public class DependenciesSolver {
             knownProjectDependencies.add(pom.getThisPom());
 
             if (interactive && packageVersion == null) {
-                String q = "Enter the upstream version for the package. If you press <Enter> it will default to " + pom.getOriginalVersion();
-                String v = userInteraction.ask(q);
+                String question = "Enter the upstream version for the package. If you press <Enter> it will default to " + pom.getOriginalVersion();
+                String v = new SimpleQuestion(question).ask();
                 if (v.isEmpty()) {
                     v = pom.getOriginalVersion();
                 }
@@ -337,7 +343,7 @@ public class DependenciesSolver {
             }
 
             if (filterModules) {
-                boolean includeModule = userInteraction.askYesNo("Include the module " + IOUtil.relativePath(baseDir, projectPom) + " ?", true);
+                boolean includeModule = new YesNoQuestion("Include the module " + IOUtil.relativePath(baseDir, projectPom) + " ?", true).ask();
                 if (!includeModule) {
                     pomTransformer.getListOfPOMs().getOrCreatePOMOptions(projectPom).setIgnore(true);
                     pomTransformer.getRulesFiles().get(IGNORE).add(DependencyRule.newToMatch(pom.getThisPom()));
@@ -356,13 +362,12 @@ public class DependenciesSolver {
                 }
             }
 
-            if (interactive && !explicitlyMentionedInRules && !"maven-plugin".equals(pom.getThisPom().getType())) {
-                Rule selectedRule = userInteraction.askForVersionRule(pom.getThisPom(), versionToRules, defaultRules);
+            if (interactive && !explicitlyMentionedInRules && !pom.getThisPom().isPlugin()) {
+                Rule selectedRule = askForVersionRule(pom.getThisPom());
                 versionToRules.put(pom.getThisPom().getVersion(), selectedRule);
                 if (selectedRule.getPattern().equals("CUSTOM")) {
-                    String s = userInteraction.ask("Enter the pattern for your custom rule (in the form s/regex/replace/)")
-                               .toLowerCase();
-                    selectedRule = new Rule(s, "My custom rule " + s);
+                    String rule = new SimpleQuestion("Enter the pattern for your custom rule (in the form s/regex/replace/)").ask().toLowerCase();
+                    selectedRule = new Rule(rule, "My custom rule " + rule);
                     defaultRules.add(selectedRule);
                 }
 
@@ -372,12 +377,12 @@ public class DependenciesSolver {
                 getRepository().registerPom(projectPom, transformedPom);
                 projectPoms.add(transformedPom.getThisPom());
 
-                if ("bundle".equals(pom.getThisPom().getType())) {
+                if (pom.getThisPom().isBundle()) {
                     String question2 = pom.getThisPom().getGroupId() + ":" + pom.getThisPom().getArtifactId() +
                             " is a bundle.\n"
                             + "Inform mh_make that dependencies of type jar which may match this library should be transformed into bundles automatically?";
 
-                    boolean transformJarsIntoBundle = userInteraction.askYesNo(question2, true);
+                    boolean transformJarsIntoBundle = new YesNoQuestion(question2, true).ask();
 
                     if (transformJarsIntoBundle) {
                         String transformBundleRule = pom.getThisPom().getGroupId() + " " + pom.getThisPom().getArtifactId()
@@ -408,7 +413,7 @@ public class DependenciesSolver {
 
             if (exploreProjects && !pom.getModules().isEmpty()) {
                 if (interactive && !askedToFilterModules) {
-                    filterModules = !userInteraction.askYesNo("This project contains modules. Include all modules?", true);
+                    filterModules = !new YesNoQuestion("This project contains modules. Include all modules?", true).ask();
                     askedToFilterModules = true;
                 }
                 for (String module : pom.getModules()) {
@@ -422,6 +427,60 @@ public class DependenciesSolver {
         }
     }
 
+    /**
+     * Asks the user to specify the substitution rule for the version.
+     * 
+     * @param dependency
+     * @return the version rule selected
+     */
+    public Rule askForVersionRule(Dependency dependency) {
+        String question = "\n"
+                + "Version of " + dependency.getGroupId() + ":"
+                + dependency.getArtifactId() + " is " + dependency.getVersion()
+                + "\nChoose how the version will be transformed:";
+
+        List<Rule> choices = getVersionRules(dependency.getVersion());       
+        
+        // select the default choice (either the previously selected rule or the 'debian' version rule)
+        int defaultChoice = 1; // the 'debian' version rule is the first one of the default rules
+        if (versionToRules.containsKey(dependency.getVersion())) {
+            Rule previouslySelectedRule = versionToRules.get(dependency.getVersion());
+            if (choices.contains(previouslySelectedRule)) {
+                defaultChoice = choices.indexOf(previouslySelectedRule);
+            }
+        }
+
+        List<String> choicesDescriptions = new ArrayList<String>();
+        for (Rule choice : choices) {
+            choicesDescriptions.add(choice.getDescription());
+        }
+
+        int choice = new ChoiceQuestion(question, defaultChoice, choicesDescriptions).ask();
+        return choices.get(choice);
+    }
+
+    /**
+     * Returns the substitution rules for the specified version.
+     */
+    private List<Rule> getVersionRules(String version) {
+        List<Rule> rules = new ArrayList<Rule>();
+
+        // add the 1.0 -> 1.x rule
+        Pattern p = Pattern.compile("(\\d+)(\\..*)");
+        Matcher matcher = p.matcher(version);
+        if (matcher.matches()) {
+            String mainVersion = matcher.group(1);
+            Rule mainVersionRule = new Rule("s/" + mainVersion + "\\..*/" + mainVersion + ".x/",
+                    "Replace all versions starting by " + mainVersion + ". with " + mainVersion + ".x");
+            rules.add(mainVersionRule);
+        }
+        
+        // add the default rules
+        rules.addAll(defaultRules);
+        
+        return rules;
+    }
+
     private POMInfo getPOM(File projectPom) throws XMLStreamException, IOException {
         POMInfo info = pomInfoCache.get(projectPom.getAbsolutePath());
         if (info != null) {
@@ -456,7 +515,7 @@ public class DependenciesSolver {
     private static boolean canBeSkippedBecauseAntIsUsedForPackaging(Dependency thisPom, String packageType, Dependency dependency, boolean runTests, boolean verbose) {
         if (!packageType.equals("ant")) return false;
 
-        if ("maven-plugin".equals(dependency.getType()) && !thisPom.getType().equals("pom")) {
+        if (dependency.isPlugin() && !thisPom.isPom()) {
             if(verbose) System.out.println("[skipped - Maven plugins are not used during a build with Ant]");
             return true;
         }
@@ -559,7 +618,7 @@ public class DependenciesSolver {
             }
         }
 
-        if (pom == null && "maven-plugin".equals(dependency.getType())) {
+        if (pom == null && dependency.isPlugin()) {
             List<POMInfo> matchingPoms = getRepository().searchMatchingPOMsIgnoreVersion(dependency);
             if (matchingPoms.size() > 1) {
                 issues.add(sourcePomLoc + ": More than one version matches the plugin " + dependency.getGroupId() + ":"
@@ -577,7 +636,7 @@ public class DependenciesSolver {
             if (management) {
                 if (verbose) System.out.println("[skipped dependency or plugin management]");
                 return null;
-            } else if ("maven-plugin".equals(dependency.getType()) && packageType.equals("ant")) {
+            } else if (dependency.isPlugin() && packageType.equals("ant")) {
                 if (verbose) System.out.println("[skipped - not used in Ant build]");
                 return null;
             }
@@ -596,7 +655,7 @@ public class DependenciesSolver {
         // libraries from jar type into bundle types, and apply as well the version substitution (for example to 2.x)
         // for Debian.
         //
-        if (pom == null && "jar".equals(dependency.getType())) {
+        if (pom == null && dependency.isJar()) {
             if (verbose) System.out.println("[check dependency with bundle type]");
 
             Dependency bundleDependency = dependency.builder().setType("bundle").build();
@@ -656,10 +715,10 @@ public class DependenciesSolver {
             if (interactive && pkg == null) {
                 pkg = scanner.searchPkgContainingJar(dependency);
                 if (pkg != null) {
-                    String q = "[error] Package " + pkg + " does not contain Maven dependency " + dependency + " but there seem to be a match\n"
+                    String question = "[error] Package " + pkg + " does not contain Maven dependency " + dependency + " but there seem to be a match\n"
                      + "If the package contains already Maven artifacts but the names don't match, try to enter a substitution rule\n"
                      + "of the form s/groupId/newGroupId/ s/artifactId/newArtifactId/ jar s/version/newVersion/ here:";
-                    String newRule = userInteraction.ask(q);
+                    String newRule = new SimpleQuestion(question).ask();
                     if (!newRule.isEmpty()) {
                         DependencyRule userRule = new DependencyRule(newRule);
                         pomTransformer.getRulesFiles().get(RULES).add(userRule);
@@ -667,16 +726,16 @@ public class DependenciesSolver {
                         return resolveDependency(dependency.applyRules(Arrays.asList(userRule)), sourcePom, buildTime, mavenExtension, management, false);
                     }
                 } else {
-                    String newRule = userInteraction.ask(
+                    String newRule = new SimpleQuestion(
                             "[error] Cannot resolve Maven dependency " + dependency + ". If you know a package that contains a compatible dependency,\n"
-                          + "try to enter a substitution rule of the form s/groupId/newGroupId/ s/artifactId/newArtifactId/ jar s/version/newVersion/ here:\n");
+                          + "try to enter a substitution rule of the form s/groupId/newGroupId/ s/artifactId/newArtifactId/ jar s/version/newVersion/ here:\n").ask();
                     while (!newRule.isEmpty()) {
                         DependencyRule userRule = new DependencyRule(newRule);
                         Dependency newDependency = dependency.applyRules(Arrays.asList(userRule));
                         if (newDependency.equals(dependency)) {
-                            newRule = userInteraction.ask("Your rule doesn't seem to apply on " + dependency
+                            newRule = new SimpleQuestion("Your rule doesn't seem to apply on " + dependency
                              + "Please enter a substitution rule of the form s/groupId/newGroupId/ s/artifactId/newArtifactId/ jar s/version/newVersion/ here,"
-                             + "or press <Enter> to give up");
+                             + "or press <Enter> to give up").ask();
                         } else {
                             pomTransformer.getRulesFiles().get(RULES).add(userRule);
                             System.out.println("Rescanning /usr/share/maven-repo...");
@@ -687,7 +746,7 @@ public class DependenciesSolver {
                 }
             }
 
-            if (interactive && userInteraction.askYesNo("Try again to resolve the dependency?", true)) {
+            if (interactive && new YesNoQuestion("Try again to resolve the dependency?", true).ask()) {
                 System.out.println("Rescanning /usr/share/maven-repo...");
                 pomTransformer.getRepository().scan();
                 // Clear caches
@@ -703,7 +762,7 @@ public class DependenciesSolver {
         // Handle the case of Maven plugins built and used in a multi-module build:
         // they need to be added to maven.cleanIgnoreRules to avoid errors during
         // a mvn clean
-        if ("maven-plugin".equals(dependency.getType()) && containsDependencyIgnoreVersion(projectPoms, dependency)) {
+        if (dependency.isPlugin() && containsDependencyIgnoreVersion(projectPoms, dependency)) {
             String ruleDef = dependency.getGroupId() + " " + dependency.getArtifactId() + " maven-plugin *";
             pomTransformer.getRulesFiles().get(CLEAN).add(new DependencyRule(ruleDef));
         }
@@ -726,7 +785,7 @@ public class DependenciesSolver {
                 if (buildTime) {
                     if ("test".equals(dependency.getScope())) {
                         debianDeps.add(TEST, libraryWithVersionConstraint);
-                    } else if ("maven-plugin".equals(dependency.getType())) {
+                    } else if (dependency.isPlugin()) {
                         if (!packageType.equals("ant")) {
                             debianDeps.add(COMPILE, libraryWithVersionConstraint);
                         }
@@ -904,7 +963,7 @@ public class DependenciesSolver {
         solver.saveSubstvars();
 
         if (!solver.issues.isEmpty()) {
-            System.err.println("Some problems where found in this project, exiting...");
+            System.err.println("Some problems were found in this project, exiting...");
             System.exit(1);
         }
     }
